/***************************************************************************
 *   Copyright (C) 2009 by Rico Rommel   *
 *   rico@bierrommel.de   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "devinterface.h"
#include <QFont>
#include <QDBusReply>


 DeviceInterface::DeviceInterface(QString name, QWidget *parent)
{
	setParent ( parent);

    	CheckButton = new QCheckButton(name, this);
	connect(CheckButton,SIGNAL(stateChanged ( bool )), this, SLOT(CheckButtonChanged( bool )));

	idlestates << "idle" << "idle_dim" << "idle_prelock" << "lock" << "suspend" << "awake";

	ComboBox = new QComboBox(this);
	ComboBox->addItems(idlestates);


}

void DeviceInterface::resizeEvent( QResizeEvent *event)
{
	int wx = event->size().width()/2;
	int wy = event->size().height();
	
	QFont font;
	float fsize = float(wy*20/100);
	font = CheckButton->font();
	font.setPointSize(fsize);

    	CheckButton->setGeometry(0,0,wx,wy);
	CheckButton->setFont(font);

    	ComboBox->setGeometry(wx,0,wx,wy);
	ComboBox->setFont(font);


}

/*!
    \fn DeviceInterface::setDBusArguments()
 */
void DeviceInterface::setDBusArguments(QString service, QString path, QString interface, QString meth, QString arg, QStringList match)
{

	dbusinterface = new QDBusInterface(service, path, interface, QDBusConnection::systemBus(), this);;
	DBusMethod = meth;
	CallArgument = arg;
	DBusMatch = match;
	

}


/*!
    \fn DeviceInterface::getPreventState()
 */
QString DeviceInterface::getPreventState()
{
	QString state;
	state = "none";

	if (CheckButton->isChecked() ) {
	QDBusMessage reply;

	
		if (CallArgument.isEmpty ()) reply = dbusinterface->call(DBusMethod);
		else reply = dbusinterface->call(DBusMethod,CallArgument);

	QString msg = reply.arguments().at(0).toString();

		if (DBusMatch.contains(msg, Qt::CaseInsensitive))
			state = ComboBox->currentText ();
	
	}
	return state;

}


void DeviceInterface::CheckButtonChanged( bool state)
{
    	if (state == true) ComboBox->setDisabled(false);
	else ComboBox->setDisabled(true);

}


 DeviceInterface::~DeviceInterface()
{
    /// @todo implement me
}





/*!
    \fn DeviceInterface::saveSettings(QSettings *settings)
 */
void DeviceInterface::saveSettings(QSettings *settings)
{
    	settings->beginGroup(CheckButton->text());
	settings->setValue("active",CheckButton->isChecked ());
	settings->setValue("state",ComboBox->currentText ());
	settings->endGroup();

}


/*!
    \fn DeviceInterface::loadSettings( QSettings *settings)
 */
void DeviceInterface::loadSettings( QSettings *settings)
{
	QString state;
	int n;
	bool checkstate;

   	settings->beginGroup(CheckButton->text());
	checkstate = settings->value("active").toBool();
	state = settings->value("state").toString();
	settings->endGroup();

	CheckButton->setCheckState (checkstate);
	if (checkstate == false) ComboBox->setDisabled(true);

	n = ComboBox->findText( state,Qt::MatchExactly | Qt::MatchCaseSensitive );
	ComboBox->setCurrentIndex(n);

}
