/***************************************************************************
 *   Copyright (C) 2009 by Rico Rommel   *
 *   rico@bierrommel.de   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "idlehandler.h"
#include <QMessageBox>
#include <QIcon>
#include <QPushButton>
#include <QWidget>
#include <QFont>
#include "devinterface.h"
#include "qcheckbutton.h"

IdleHandler::IdleHandler()
{
	QIcon icon("/usr/share/idlehandler/idlehandler.png");
	setWindowIcon(icon); 
	systrayicon = new QSystemTrayIcon(icon, this);

	connect(systrayicon, SIGNAL(activated(QSystemTrayIcon::ActivationReason)),this, SLOT(iconActivated(QSystemTrayIcon::ActivationReason)));
	systrayicon->show();

	QuitButton = new QPushButton(QObject::tr("Quit"),this);
	connect(QuitButton,SIGNAL(clicked()), qApp, SLOT(quit()));

	CloseButton = new QPushButton(QObject::tr("Close"),this);
	connect(CloseButton,SIGNAL(clicked()), this, SLOT(close()));

	SaveButton = new QPushButton(QObject::tr("Save"),this);
	connect(SaveButton,SIGNAL(clicked()), this, SLOT(saveSettings()));


	tabWidget[0] = new QWidget(this);
	tabWidget[1] = new QWidget(this);
	tabWidget[2] = new QWidget(this);
	tabView = new QTabWidget(this);
	tabView->addTab(tabWidget[GENTAB],QObject::tr("general"));
	tabView->addTab(tabWidget[APPTAB],QObject::tr("applications"));
	tabView->addTab(tabWidget[BRITAB],QObject::tr("timeout/brightness"));
	
	makeDBUSConnection();


	idlestates << "busy" << "idle" << "idle_dim" << "idle_prelock" << "lock" << "suspend" << "awake";


    	settings = new QSettings("openmoko","idlehandler");
	settings->sync ();

	createUSBInterface();
	createGPSInterface();
	createCALLInterface();
	createAppInterface(tabWidget[APPTAB]);

	

	createStatesettings();



	

}


IdleHandler::~IdleHandler()
{
//    QDBusConnection::disconnectFromBus("dbus");
}


void IdleHandler::iconActivated(QSystemTrayIcon::ActivationReason)
{
    showMaximized ();
}


void IdleHandler::closeEvent ( QCloseEvent * event )
{
     	hide();
        event->ignore();
}





void IdleHandler::resizeEvent( QResizeEvent *event)
{
	int wx = event->size().width();
	int wy = event->size().height();
	
	int space = wx * SPACE / 100;
	int w = wx * WIDTH / 100;
	int h = wy * HEIGHT / 100;

	QFont font;
	float fsize = float(h*20/100);
	font = CloseButton->font();
	font.setPointSize(fsize);


    	CloseButton->setGeometry(space,wy-h-space,w,h);
	CloseButton->setFont(font);
	QuitButton->setGeometry(space+w,wy-h-space,w,h);
	QuitButton->setFont(font);
	SaveButton->setGeometry(space,wy-2*h-space,2*w,h);
	SaveButton->setFont(font);

	tabView->setGeometry(0,0, wx, wy - 2*h + 3*space);

	wx = tabView->width();
	wy = tabView->height() - 50;
	
	space = wx * SPACE / 100;
	w = wx * WIDTH / 100;
	h = wy * CHECKBUTTONHEIGHT / 100;


	USB->setGeometry(space,space,2*w,h);
	GPS->setGeometry(space,2*space+h,2*w,h);
	CALL->setGeometry(space,3*space+2*h,2*w,h);

	int n;

	for (n=0; n<=3; n++) {
		APP[n]->setGeometry(space,(n+1)*space+n*h,2*w,h);
	}


	h =  SETTINGSLABELHEIGHT*wy/100;

	SettingsTableHead[0]->setGeometry(0,space, wx * SETTINGSSTATEWIDTH/100, h);
	SettingsTableHead[1]->setGeometry(wx * SETTINGSSTATEWIDTH/100 + space, space, wx * SETTINGSTIMEOUTWIDTH/100, h);
	SettingsTableHead[2]->setGeometry(wx * (SETTINGSTIMEOUTWIDTH + SETTINGSSTATEWIDTH)/100 + 2 *space, space, wx * SETTINGSTIMEOUTWIDTH/100, h);

	fsize = float(h*30/100);
	font.setPointSize(fsize);
	font.setWeight(70);

	for (n=0; n<=2; n++) SettingsTableHead[n]->setFont(font);


	for (n=0; n<= 5; n++)
	StateSettings[n]->setGeometry(0,(n+1)*h+(n+2)*space,wx,h);
	

}


void IdleHandler::IdleStateChanged(QString state)
{
	qDebug() << "receiving Signal " << state; 
	int n;

	bool goback = false;
	if (state == USB->getPreventState()) goback = true ;
	if (state == GPS->getPreventState()) goback = true ;
	if (state == CALL->getPreventState()) goback = true ;
	
	for (n=0; n<=3; n++) {
		if (state == APP[n]->getPreventState()) goback = true ;
	}

	n = idlestates.indexOf( state, 0);

	if (goback) {
		n = n -1;
		idleinterface->call("SetState",idlestates.at(n));
		qDebug() << "Sending Signal " << idlestates.at(n); 

	
		}
	else {
		if (state == "suspend") usageinterface->call("Suspend");
		qDebug() << "Sending Signal " << state;
		
		}

	QDBusMessage reply = displayinterface->call("SetBrightness", StateSettings[n]->brightness());
	qDebug() << "setting brightness: " << StateSettings[n]->brightness();
		

}


void IdleHandler::createUSBInterface()
{
    	USB = new DeviceInterface(QObject::tr("USB"), tabView->widget(GENTAB));

	QStringList match; 
	match << "full" << "charging";

	USB->setDBusArguments("org.freesmartphone.odeviced", "/org/freesmartphone/Device/PowerSupply/battery", "org.freesmartphone.Device.PowerSupply","GetPowerStatus","", match);
	USB->loadSettings( settings );

}


void IdleHandler::createGPSInterface()
{
	GPS = new DeviceInterface(QObject::tr("GPS"), tabView->widget(GENTAB));

	QStringList match; 
	match << "true";

	GPS->setDBusArguments("org.freesmartphone.ousaged", "/org/freesmartphone/Usage", "org.freesmartphone.Usage","GetResourceState","GPS", match);
	GPS->loadSettings( settings );

}



void IdleHandler::createCALLInterface()
{
    	CALL = new DeviceInterface(QObject::tr("Call"), tabView->widget(GENTAB));

	QStringList match; 
	match << "gsmhandset";

	CALL->setDBusArguments("org.freesmartphone.odeviced", "/org/freesmartphone/Device/Audio", "org.freesmartphone.Device.Audio","GetScenario","", match);

	CALL->loadSettings( settings );

}


/*!
    \fn IdleHandler::saveSettings()
 */
void IdleHandler::saveSettings()
{
	USB->saveSettings( settings );
	GPS->saveSettings( settings );
	CALL->saveSettings( settings );

	int n;

	for (n=0;n<=3;n++) APP[n]->saveSettings( settings );

	for (n=0; n<=5; n++) StateSettings[n]->saveSettings( settings);

	settings->sync ();   
}



void IdleHandler::createAppInterface( QWidget *tab)
{
	int n;
	for (n = 0; n<=3; n++) {
       	APP[n] = new AppInterface(QObject::tr("Application") + QString::number(n,10), tab);
	APP[n]->loadSettings( settings );
	}
}


/*!
    \fn IdleHandler::makeDBUSConnection()
 */
void IdleHandler::makeDBUSConnection()
{
  	do {
		idleinterface = new QDBusInterface("org.freesmartphone.odeviced", "/org/freesmartphone/Device/IdleNotifier/0", "org.freesmartphone.Device.IdleNotifier", QDBusConnection::systemBus(), this);
		if (!idleinterface->isValid ()) sleep(5);
		}
	while (!idleinterface->isValid ());
	qDebug() << "idlenotifier connected";

	connect(idleinterface,SIGNAL( State(QString) ), this, SLOT(IdleStateChanged(QString)));

	do {
		usageinterface = new QDBusInterface("org.freesmartphone.ousaged", "/org/freesmartphone/Usage", "org.freesmartphone.Usage", QDBusConnection::systemBus(), this);
		if (!usageinterface->isValid ()) sleep(5);
		}
	while (!usageinterface->isValid ());
	qDebug() << "ousaged connected";

	do {
		displayinterface = new QDBusInterface("org.freesmartphone.odeviced", "/org/freesmartphone/Device/Display/0", "org.freesmartphone.Device.Display", QDBusConnection::systemBus(), this);
		if (!idleinterface->isValid ()) sleep(5);
		}
	while (!idleinterface->isValid ());
	qDebug() << "display connected";
}

void IdleHandler::createStatesettings()
{
	SettingsTableHead[0] = new QLabel(QObject::tr("idlestate"), tabWidget[BRITAB]);
	SettingsTableHead[1] = new QLabel(QObject::tr("time"), tabWidget[BRITAB]);
	SettingsTableHead[2] = new QLabel(QObject::tr("bright"), tabWidget[BRITAB]);
	int n;

	for (n=0; n<=2; n++) SettingsTableHead[n]->setFrameShape(QFrame::Box);

	for (n=0; n<=5; n++) {
	StateSettings[n] = new QStateSettings(idlestates.at(n),	tabWidget[BRITAB], idleinterface);
	StateSettings[n]->loadSettings( settings );
	}

	StateSettings[0]->disableTimeout();
	StateSettings[5]->disableBrightness();

	StateSettings[0]->setTimeoutMinimum( 1 );
	StateSettings[0]->setBrightnessMinimum( 50 );

}


